package java.io;

import java.util.Arrays;

public class ByteArrayOutputStream extends OutputStream
{
    protected byte[] buf;
    protected int count;
    private static final int MAX_ARRAY_SIZE = 2147483639;
    
    public ByteArrayOutputStream() {
        this(32);
    }
    
    public ByteArrayOutputStream(int i) {
        if (i < 0) {
            throw new IllegalArgumentException("Negative initial size: " + i);
        }
        this.buf = new byte[i];
        return;
    }
    
    private void ensureCapacity(int n) {
        if (n - this.buf.length > 0) {
            this.grow(n);
        }
    }
    
    private void grow(int n) {
        int hugeCapacity = this.buf.length << 1;
        if (hugeCapacity - n < 0) {
            hugeCapacity = n;
        }
        if (hugeCapacity - 2147483639 > 0) {
            hugeCapacity = hugeCapacity(n);
        }
        this.buf = Arrays.copyOf(this.buf, hugeCapacity);
    }
    
    private static int hugeCapacity(int n) {
        if (n < 0) {
            throw new OutOfMemoryError();
        }
        return (n > 2147483639) ? Integer.MAX_VALUE : 2147483639;
    }
    
    @Override
    public synchronized void write(int n) {
        this.ensureCapacity(this.count + 1);
        this.buf[this.count] = (byte)n;
        ++this.count;
    }
    
    @Override
    public synchronized void write(byte[] array, int n, int n2) {
        if (n < 0 || n > array.length || n2 < 0 || n + n2 - array.length > 0) {
            throw new IndexOutOfBoundsException();
        }
        this.ensureCapacity(this.count + n2);
        System.arraycopy(array, n, this.buf, this.count, n2);
        this.count += n2;
        return;
    }
    
    public synchronized void writeTo(OutputStream outputStream) throws IOException {
        outputStream.write(this.buf, 0, this.count);
    }
    
    public synchronized void reset() {
        this.count = 0;
    }
    
    public synchronized byte[] toByteArray() {
        return Arrays.copyOf(this.buf, this.count);
    }
    
    public synchronized int size() {
        return this.count;
    }
    
    @Override
    public synchronized String toString() {
        return new String(this.buf, 0, this.count);
    }
    
    public synchronized String toString(String charsetName) throws UnsupportedEncodingException {
        return new String(this.buf, 0, this.count, charsetName);
    }
    
    @Deprecated
    public synchronized String toString(int hibyte) {
        return new String(this.buf, hibyte, 0, this.count);
    }
    
    @Override
    public void close() throws IOException {
    }
}
